import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { CreateTerritoryCommand } from '../../commands/create-territory.command';
import { Messager } from '../../adapters/secondaries/messager';
import { Territory } from '../entities/territory';
import { CreateTerritoryRequest } from '../dtos/create-territory.request';
import { TerritoryLoggingPresenter } from '../../adapters/secondaries/territory-logging.presenter';
import { TerritoryPresenter } from '../../adapters/secondaries/territory.presenter';

@CommandHandler(CreateTerritoryCommand)
export class CreateTerritoryUseCase {
  constructor(
    private readonly repository: TerritoriesRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (command: CreateTerritoryCommand): Promise<Territory> => {
    const entity = this.mapper.map(
      command.createTerritoryRequest,
      CreateTerritoryRequest,
      Territory,
    );

    try {
      const territory = await this.repository.createTerritory(entity);
      this.messager.publish(
        'territory.create',
        JSON.stringify(
          this.mapper.map(territory, Territory, TerritoryPresenter),
        ),
      );
      this.messager.publish(
        'logging.territory.create.info',
        JSON.stringify(
          this.mapper.map(territory, Territory, TerritoryLoggingPresenter),
        ),
      );
      return territory;
    } catch (error) {
      let key = 'logging.territory.create.crit';
      if (error.message.includes('already exists')) {
        key = 'logging.territory.create.warning';
      }
      this.messager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
