import { QueryHandler } from '@nestjs/cqrs';
import { ICollection } from 'src/modules/database/interfaces/collection.interface';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { FindAllTerritoriesForPointQuery } from '../../queries/find-all-territories-for-point.query';
import { Territory } from '../entities/territory';
import { Point } from '../entities/point';

@QueryHandler(FindAllTerritoriesForPointQuery)
export class FindAllTerritoriesForPointUseCase {
  constructor(private readonly repository: TerritoriesRepository) {}

  execute = async (
    findAllTerritoriesForPointQuery: FindAllTerritoriesForPointQuery,
  ): Promise<ICollection<Territory>> =>
    this.repository.findForPoint(
      new Point(
        findAllTerritoriesForPointQuery.point.lon,
        findAllTerritoriesForPointQuery.point.lat,
      ),
    );
}
